@@ -1,7 +1,7 @@
 package App::ProcUtils;
 
 our $DATE = '2019-09-11'; # DATE
-our $VERSION = '0.035'; # VERSION
+our $VERSION = '0.036'; # VERSION
 
 use 5.010001;
 use strict;
@@ -10,6 +10,125 @@ use Log::ger;
 
 our %SPEC;
 
+our %args_filtering = (
+    cmdline_match => {
+        schema => 're*',
+        tags => ['category:filtering'],
+        pos => 0,
+    },
+    cmdline_not_match => {
+        schema => 're*',
+        tags => ['category:filtering'],
+    },
+    exec_match => {
+        schema => 're*',
+        tags => ['category:filtering'],
+    },
+    exec_not_match => {
+        schema => 're*',
+        tags => ['category:filtering'],
+    },
+    pids => {
+        'x.name.is_plural' => 1,
+        'x.name.singular' => 'pid',
+        schema => ['array*', of=>'unix::pid*'],
+        tags => ['category:filtering'],
+    },
+    uids => {
+        'x.name.is_plural' => 1,
+        'x.name.singular' => 'uid',
+        schema => ['array*', of=>'unix::local_uid*'],
+        tags => ['category:filtering'],
+    },
+    logic => {
+        schema => ['str*', in=>['AND','OR']],
+        default => 'AND',
+        cmdline_aliases => {
+            and => {is_flag=>0, summary=>'Shortcut for --logic=AND', code=>sub {$_[0]{logic} = 'AND' }},
+            or  => {is_flag=>0, summary=>'Shortcut for --logic=OR' , code=>sub {$_[0]{logic} = 'OR'  }},
+        },
+        tags => ['category:filtering'],
+    },
+    code => {
+        schema => 'code*',
+        description => <<'_',
+
+Code is given <pm:Proc::ProcessTable::Process> object, which is a hashref
+containing items like `pid`, `uid`, etc. It should return true to mean that a
+process matches.
+
+_
+        tags => ['category:filtering'],
+    },
+);
+
+our %arg_detail = (
+    detail => {
+        summary => 'Return detailed records instead of just PIDs',
+        schema => 'true',
+        cmdline_aliases=>{l=>{}},
+    },
+);
+
+our %arg_quiet = (
+    quiet => {
+        schema => 'true',
+        cmdline_aliases=>{q=>{}},
+    },
+);
+
+our @proc_fields = (
+    # follows the order of 'ps aux'
+    "uid",
+    "pid",
+    "pctcpu",
+    "pctmem",
+    "size",
+    "rss",
+    "ttydev",
+    "ttynum",
+    "state",
+    "start",
+    "time",
+    "cmndline",
+
+    "cmajflt",
+    "cminflt",
+    "cstime",
+    "ctime",
+    "cutime",
+    "cwd",
+    "egid",
+    "euid",
+    "exec",
+    "fgid",
+    "flags",
+    "fname",
+    "fuid",
+    "gid",
+    "majflt",
+    "minflt",
+    "pgrp",
+    "ppid",
+    "priority",
+    "sess",
+    "sgid",
+    "stime",
+    "suid",
+    "utime",
+    "wchan",
+
+    # not included
+    # environ
+);
+
+sub _proc_obj_to_hash {
+    my $row = {%{$_[0]}};
+    $row->{cmdline} = join(" ", grep {$_ ne ''} @{ $row->{cmdline} });
+    delete $row->{environ};
+    $row;
+}
+
 $SPEC{list_parents} = {
     v => 1.1,
     summary => 'List all the parents of the current process',
@@ -29,130 +148,23 @@ sub table {
 
     my $t = Proc::ProcessTable->new;
 
-    my $resmeta = {
-        'table.fields' => [
-            # follows the order of 'ps aux'
-            "uid",
-            "pid",
-            "pctcpu",
-            "pctmem",
-            "size",
-            "rss",
-            "ttydev",
-            "ttynum",
-            "state",
-            "start",
-            "time",
-            "cmndline",
-
-            "cmajflt",
-            "cminflt",
-            "cstime",
-            "ctime",
-            "cutime",
-            "cwd",
-            "egid",
-            "euid",
-            "exec",
-            "fgid",
-            "flags",
-            "fname",
-            "fuid",
-            "gid",
-            "majflt",
-            "minflt",
-            "pgrp",
-            "ppid",
-            "priority",
-            "sess",
-            "sgid",
-            "stime",
-            "suid",
-            "utime",
-            "wchan",
-
-            # not included
-            # environ
-        ],
-    };
+    my $resmeta = {};
+    $resmeta->{'table.fields'} = \@proc_fields;
 
     my @rows;
     for my $p (@{ $t->table }) {
-        my $row = {%$p};
-        $row->{cmdline} = join(" ", grep {$_ ne ''} @{ $row->{cmdline} });
-        delete $row->{environ};
+        my $row = _proc_obj_to_hash($p);
         push @rows, $row;
     }
 
     [200, "OK", \@rows, $resmeta];
 }
 
-$SPEC{kill} = {
-    v => 1.1,
-    summary => 'Kill processes that match criteria',
-    args => {
-        signal => {
-            schema => 'unix::signal*',
-            default => 'TERM',
-        },
-        cmdline_match => {
-            schema => 're*',
-            tags => ['category:filtering'],
-            pos => 0,
-        },
-        cmdline_not_match => {
-            schema => 're*',
-            tags => ['category:filtering'],
-        },
-        exec_match => {
-            schema => 're*',
-            tags => ['category:filtering'],
-        },
-        exec_not_match => {
-            schema => 're*',
-            tags => ['category:filtering'],
-        },
-        pids => {
-            'x.name.is_plural' => 1,
-            'x.name.singular' => 'pid',
-            schema => ['array*', of=>'unix::pid*'],
-            tags => ['category:filtering'],
-        },
-        uids => {
-            'x.name.is_plural' => 1,
-            'x.name.singular' => 'uid',
-            schema => ['array*', of=>'unix::local_uid*'],
-            tags => ['category:filtering'],
-        },
-        logic => {
-            schema => ['str*', in=>['AND','OR']],
-            default => 'AND',
-            cmdline_aliases => {
-                and => {is_flag=>0, summary=>'Shortcut for --logic=AND', code=>sub {$_[0]{logic} = 'AND' }},
-                or  => {is_flag=>0, summary=>'Shortcut for --logic=OR' , code=>sub {$_[0]{logic} = 'OR'  }},
-            },
-            tags => ['category:filtering'],
-        },
-        code => {
-            schema => 'code*',
-            description => <<'_',
-
-Code is given <pm:Proc::ProcessTable::Process> object, which is a hashref
-containing items like `pid`, `uid`, etc. It should return true to signal that a
-process matches and should be killed.
-
-_
-            tags => ['category:filtering'],
-        },
-    },
-    features => {
-        dry_run => 1,
-    },
-};
-sub kill {
+sub _kill_or_list {
     require Proc::ProcessTable;
     require String::Truncate;
 
+    my $which = shift;
     my %args = @_;
 
     my $is_or = ($args{logic} // 'AND') eq 'OR' ? 1:0;
@@ -220,15 +232,79 @@ sub kill {
         }
 
       MATCH:
-        if ($args{-dry_run}) {
-            log_info "[DRY-RUN] Sending %s signal to PID %d (%s) ...",
-                $args{signal}, $proc_entry->{pid}, String::Truncate::elide($cmdline, 40, {truncate=>'middle'});
+        if ($which eq 'kill') {
+            if ($args{-dry_run}) {
+                log_info "[DRY-RUN] Sending %s signal to PID %d (%s) ...",
+                    $args{signal}, $proc_entry->{pid}, String::Truncate::elide($cmdline, 40, {truncate=>'middle'});
+            } else {
+                kill $args{signal} => $proc_entry->{pid};
+            }
         } else {
-            kill $args{signal} => $proc_entry->{pid};
+            push @proc_matches, _proc_obj_to_hash($proc_entry);
         }
-    }
+    } # for each entry
 
-    [200, "OK", ];
+    if ($which eq 'kill') {
+        return [200, "OK"];
+    } else {
+        my $resmeta = {};
+        if ($args{detail}) {
+            $resmeta->{'table.fields'} = \@proc_fields;
+        } else {
+            @proc_matches = map { $_->{pid} } @proc_matches;
+        }
+        return [200, "OK", \@proc_matches, $resmeta];
+    }
+}
+
+$SPEC{kill} = {
+    v => 1.1,
+    summary => 'Kill processes that match criteria',
+    args => {
+        signal => {
+            schema => 'unix::signal*',
+            default => 'TERM',
+        },
+        %args_filtering,
+    },
+    features => {
+        dry_run => 1,
+    },
+};
+sub kill {
+    _kill_or_list('kill', @_);
+}
+
+$SPEC{list} = {
+    v => 1.1,
+    summary => 'List processes that match criteria',
+    args => {
+        %args_filtering,
+        %arg_detail,
+    },
+};
+sub list {
+    _kill_or_list('list', @_);
+}
+
+$SPEC{exists} = {
+    v => 1.1,
+    summary => 'Check if processes that match criteria exists',
+    args => {
+        %args_filtering,
+        %arg_quiet,
+    },
+};
+sub exists {
+    my %args = @_;
+    my $quiet = delete $args{quiet};
+    my $res = &list(%args);
+    return $res unless $res->[0] == 200;
+    if (@{ $res->[2] }) {
+        return [200, "OK", $quiet ? "" : "Processes that match criteria exist", {'cmdline.exit_code' => 0}];
+    } else {
+        return [200, "OK", $quiet ? "" : "Processes that match criteria DO NOT exist", {'cmdline.exit_code' => 1}];
+    }
 }
 
 1;
@@ -246,7 +322,7 @@ App::ProcUtils - Command line utilities related to processes
 
 =head1 VERSION
 
-This document describes version 0.035 of App::ProcUtils (from Perl distribution App-ProcUtils), released on 2019-09-11.
+This document describes version 0.036 of App::ProcUtils (from Perl distribution App-ProcUtils), released on 2019-09-11.
 
 =head1 SYNOPSIS
 
@@ -254,8 +330,12 @@ This distribution provides the following command-line utilities:
 
 =over
 
+=item * L<proc-exists>
+
 =item * L<proc-kill>
 
+=item * L<proc-list>
+
 =item * L<proc-list-parents>
 
 =item * L<proc-table>
@@ -265,6 +345,57 @@ This distribution provides the following command-line utilities:
 =head1 FUNCTIONS
 
 
+=head2 exists
+
+Usage:
+
+ exists(%args) -> [status, msg, payload, meta]
+
+Check if processes that match criteria exists.
+
+This function is not exported.
+
+Arguments ('*' denotes required arguments):
+
+=over 4
+
+=item * B<cmdline_match> => I<re>
+
+=item * B<cmdline_not_match> => I<re>
+
+=item * B<code> => I<code>
+
+Code is given L<Proc::ProcessTable::Process> object, which is a hashref
+containing items like C<pid>, C<uid>, etc. It should return true to mean that a
+process matches.
+
+=item * B<exec_match> => I<re>
+
+=item * B<exec_not_match> => I<re>
+
+=item * B<logic> => I<str> (default: "AND")
+
+=item * B<pids> => I<array[unix::pid]>
+
+=item * B<quiet> => I<true>
+
+=item * B<uids> => I<array[unix::local_uid]>
+
+=back
+
+Returns an enveloped result (an array).
+
+First element (status) is an integer containing HTTP status code
+(200 means OK, 4xx caller error, 5xx function error). Second element
+(msg) is a string containing error message, or 'OK' if status is
+200. Third element (payload) is optional, the actual result. Fourth
+element (meta) is called result metadata and is optional, a hash
+that contains extra information.
+
+Return value:  (any)
+
+
+
 =head2 kill
 
 Usage:
@@ -289,8 +420,8 @@ Arguments ('*' denotes required arguments):
 =item * B<code> => I<code>
 
 Code is given L<Proc::ProcessTable::Process> object, which is a hashref
-containing items like C<pid>, C<uid>, etc. It should return true to signal that a
-process matches and should be killed.
+containing items like C<pid>, C<uid>, etc. It should return true to mean that a
+process matches.
 
 =item * B<exec_match> => I<re>
 
@@ -329,6 +460,59 @@ Return value:  (any)
 
 
 
+=head2 list
+
+Usage:
+
+ list(%args) -> [status, msg, payload, meta]
+
+List processes that match criteria.
+
+This function is not exported.
+
+Arguments ('*' denotes required arguments):
+
+=over 4
+
+=item * B<cmdline_match> => I<re>
+
+=item * B<cmdline_not_match> => I<re>
+
+=item * B<code> => I<code>
+
+Code is given L<Proc::ProcessTable::Process> object, which is a hashref
+containing items like C<pid>, C<uid>, etc. It should return true to mean that a
+process matches.
+
+=item * B<detail> => I<true>
+
+Return detailed records instead of just PIDs.
+
+=item * B<exec_match> => I<re>
+
+=item * B<exec_not_match> => I<re>
+
+=item * B<logic> => I<str> (default: "AND")
+
+=item * B<pids> => I<array[unix::pid]>
+
+=item * B<uids> => I<array[unix::local_uid]>
+
+=back
+
+Returns an enveloped result (an array).
+
+First element (status) is an integer containing HTTP status code
+(200 means OK, 4xx caller error, 5xx function error). Second element
+(msg) is a string containing error message, or 'OK' if status is
+200. Third element (payload) is optional, the actual result. Fourth
+element (meta) is called result metadata and is optional, a hash
+that contains extra information.
+
+Return value:  (any)
+
+
+
 =head2 list_parents
 
 Usage:
@@ -393,6 +577,11 @@ When submitting a bug or request, please include a test-file or a
 patch to an existing test-file that illustrates the bug or desired
 feature.
 
+=head1 SEE ALSO
+
+L<Proc::Find> is a similar module; App::ProcUtils provides the CLI scripts as
+well as function interface.
+
 =head1 AUTHOR
 
 perlancar <perlancar@cpan.org>
